﻿//Code for the Power Split Device demo
//Author: Alex Hart http://eahart.com
//Contributing author: J. Kusnetz http://www.kusnetz.net/prius/

/*

Copyright (c) 2006, Alex Hart
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

    * Redistributions of source code must retain the above copyright
	notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
	notice, this list of conditions and the following disclaimer in the
	documentation and/or other materials provided with the distribution.
    * Neither the name of Alex Hart nor the names of its
	contributors may be used to endorse or promote products derived
	from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

*/


/************

Some basics about the relationships in the PSD:

 S : Sun_mc, the "Sun" or center gear, connects to MG1.
 C : PlanetsOrbit_mc, the "Planet carrier", or the 4 middle gears, which connect to ICE and move as a unit.
 R : Ring_mc, the outer ring, which connects to MG2 and vehicle traction.
 P : PlanetsOrbit_mc.PlanetX_mc, the idividual planet gear.
     Speed of "P" is insignificant to the engine, but is needed for proper visualization of the system.

Equations for rotational speed

 S = 3.6 * C - 2.6 * R
 P = 3.3913 * R - 3.3913 * C

*************/

// create the Nomograph and CarConnection objects
var myLine : DrawNomograph = new DrawNomograph( MG2Slider_mc, ICESlider_mc, MG1Slider_mc, "Handle_mc", this );
myLine.setActive( true );
var myCarConnection : CarConnection = new CarConnection( this );
var Engine : EngineAttrs = new EngineAttrs( this );

// Slider limits (set manually within the FLA by positioning some empty movieclips) and initial slider positions
var MG1UpperSliderLimit : Number = -200;
var MG1LowerSliderLimit : Number = 200;
var MG2UpperSliderLimit : Number = MG2Slider_mc.UpperLimit_mc._y;
var MG2LowerSliderLimit : Number = EngineAttrs.MG2MinRPM / EngineAttrs.MG2MaxRPM * MG2UpperSliderLimit;
var ICEUpperSliderLimit : Number = ICESlider_mc.UpperLimit_mc._y;
var MG2SliderPosition : Number = 0;
var ICESliderPosition : Number = 0;
var MG2SliderPercent : Number = 0;
var ICESliderPercent : Number = 0;

var SlidersNeedUpdate : Boolean = false;

//set engine stop point in relation to the slider
var ICEThreshold : Number = EngineAttrs.ICEMinRPM / EngineAttrs.ICEMaxRPM * ICEUpperSliderLimit;

// Settings for the checkbox component
Overlay.themeColor = "haloOrange";
Overlay.font = "Verdana";
Overlay.color = "0x666666";

//Show the min and max RPM values on the slider scale
LowRPM_mc.Label_txt.text = EngineAttrs.MG1MinRPM + " RPM";
HighRPM_mc.Label_txt.text = EngineAttrs.MG1MaxRPM + " RPM";

// This sets up a function that will run every time a new frame is drawn in the flash movie. The end of this function
// is just above the function "SetSafeICERPM" so everything in here runs once a frame (24 frames per second) to keep
// the display up to date. 

var RootLoop : MovieClip = _level0.createEmptyMovieClip("RootLoop_mc", 1001);

RootLoop.onEnterFrame = function(){
	if ( SlidersNeedUpdate || myCarConnection.getCanViewStatus() )
	{
		UpdateSliders();
	}
	Engine.updateEngine();
	UpdateDisplays();

	// Rotate the gear movie clips into their positions as calculated above
	PSDdemo_mc.Ring_mc._rotation = EngineAttrs.RingRotation % 360;
	PSDdemo_mc.PlanetsOrbit_mc._rotation = EngineAttrs.PlanetCarrierRotation % 360;
	PSDdemo_mc.Sun_mc._rotation = EngineAttrs.SunRotation;

	// Rotate the individual planet gears based on the rest of the system.
	for ( i in PSDdemo_mc.PlanetsOrbit_mc ) {
		PSDdemo_mc.PlanetsOrbit_mc[i]._rotation = ( 3.3913 * PSDdemo_mc.Ring_mc._rotation ) - ( 3.3913 * PSDdemo_mc.PlanetsOrbit_mc._rotation );
	}
}

// What to do when ICE is between 0 and min operating speed. This is set using approximate values, calculated by
// hand from the slider scale vs. the ICE rpm scale. I'll probably fix this to work automatically based on
// a single setting of "MinimumICERPM"
// Right now this pops the slider up to roughly 1000rpm if you're above 500, and down if you're below.

function SetSafeICERPM () {
	if (ICESlider_mc.Handle_mc._y <= ICEThreshold / 2 && ICESlider_mc.Handle_mc._y > ICEThreshold ) {
		ICESlider_mc.Handle_mc._y = ICEThreshold;
	} else if ( ICESlider_mc.Handle_mc._y < 0 && ICESlider_mc.Handle_mc._y > ICEThreshold / 2 ) {
		ICESlider_mc.Handle_mc._y = 0;
	}
}

// Set up the button states and drag limits for the sliders. 

function UpdateSliders() {

	/*******
	
	MG2RPM is 6500 * Slider percentage
	ICERPM is 4500 * Slider percentage
	
	Slider positions are negative because flash uses top-left as origin. so starting at  y = 0  and
	going up 200 pixels, that handle will be at  y = -200. Dividing by the upper slider limit gives
	us a positive percentage of the full slider value that we can use in the next step.
	
	*******/
	
	var lMG1RPM : Number = EngineAttrs.MG1RPM;
	var lMG2RPM : Number = EngineAttrs.MG2RPM;
	var lICERPM : Number = EngineAttrs.ICERPM;
	var lMG1MinRPM : Number = EngineAttrs.MG1MinRPM;
	var lMG1MaxRPM : Number = EngineAttrs.MG1MaxRPM;
	var lMG2MinRPM : Number = EngineAttrs.MG2MinRPM;
	var lMG2MaxRPM : Number = EngineAttrs.MG2MaxRPM;
	var lICEMaxRPM : Number = EngineAttrs.ICEMaxRPM;
	var lMPH : Number = EngineAttrs.MPH;
	
	if ( myCarConnection.getCanViewStatus() ) {
		// Update Slider positions
        MG2Slider_mc.Handle_mc._y = lMG2RPM / lMG2MaxRPM * MG2UpperSliderLimit;
        ICESlider_mc.Handle_mc._y = lICERPM / lICEMaxRPM * ICEUpperSliderLimit;
	}

	MG2SliderPosition = MG2Slider_mc.Handle_mc._y;
	MG2SliderPercent = MG2SliderPosition / MG2UpperSliderLimit;
	ICESliderPosition = ICESlider_mc.Handle_mc._y;
	ICESliderPercent = ICESliderPosition / ICEUpperSliderLimit;
	
	//update MG2RPM and ICERPM if the CAN-view is not connected:
	if ( !myCarConnection.getCanViewStatus() ) {
		EngineAttrs.MG2RPM = 6500 * MG2SliderPercent;
		EngineAttrs.ICERPM = 4500 * ICESliderPercent;
		//update MG1Speed as well
		lMG1RPM = Engine.updateMG1("nocanview");
	}
	
	// Set the position of the MG1 slider
	// Set warning state for MG1 if its speed is out of range.

	if ( lMG1RPM > lMG1MaxRPM ){
		MG1RPM_txt.textColor = "0xFF0000";
		myLine.setWarning( true );
		MG1Slider_mc.Handle_mc._y = MG1UpperSliderLimit;
	} else if ( lMG1RPM < lMG1MinRPM ){
		MG1RPM_txt.textColor = "0xFF0000";
		myLine.setWarning( true );
		MG1Slider_mc.Handle_mc._y = MG1LowerSliderLimit;;
	} else {
		MG1RPM_txt.textColor = "0x01B1B1";
		myLine.setWarning( false );
		MG1Slider_mc.Handle_mc._y = lMG1RPM / lMG1MaxRPM * MG1UpperSliderLimit;
	}

	//  Fill in values for all the displays.
	MG2RPM_txt.text = Math.round( lMG2RPM );
	ICERPM_txt.text = Math.round( lICERPM );
	MG1RPM_txt.text = Math.round( lMG1RPM );
	MPH_txt.text = lMPH;
	
	//Update Nomograph NOW
	if (Overlay.selected)
	{
		myLine.updatePoints();
	}

	// Set warning state for ICE speed. *rough value* is 1k RPM so it would be nice to get real values for when ICE starts and stops.
	
	if ( lICERPM > 0 && lICERPM < lICEMinRPM-5 ){ //-5 gives a little wiggle room due to imperfect pixel positioning
		ICERPM_txt.textColor = "0xFF0000";
	} else {
		ICERPM_txt.textColor = "0xEC8E02";
	}
	
	if ( myCarConnection.getCarMode() )
	{
		myLine.setLineProperties( 0xFFFFFF, 100, 2 );
	}
}

MPGBar_mc._visible = false;
SOCBar_mc._visible = false;

var MPGRed : Number = 255;
var MPGGreen : Number = 255;
var MPGBlue : Number = 0;
import flash.geom.ColorTransform;
import flash.geom.Transform;
var SOCColorTrans : ColorTransform = new ColorTransform();
var MPGColorTrans : ColorTransform = new ColorTransform(0, 0, 0, 1, MPGRed, MPGGreen, MPGBlue, 0);
var SOCTrans : Transform = new Transform( SOCBar_mc );
var MPGTrans : Transform = new Transform( MPGBar_mc );

function UpdateDisplays () {

	//if Carmode, show MPG and SOC bars.
	if ( myCarConnection.getCarMode() )
	{
		MPGBar_mc._visible = true;
		SOCBar_mc._visible = true;
	}
	
	//Scale the MPG bar when MPG is between 0-100.
	//Show values larger than 100, but "Stealth" when MPG is 255
	if ( Math.round(EngineAttrs.MPG) <= 100 ) {
		MPGBar_mc._xscale = EngineAttrs.MPG;
		MPGDisplay_mc.MPG_txt.text = Math.round( EngineAttrs.MPG );
	} else if ( Math.round(EngineAttrs.MPG) > 100 && Math.round(EngineAttrs.MPG) < 255 ) {
		MPGBar_mc._xscale = 100;
		MPGDisplay_mc.MPG_txt.text = Math.round( EngineAttrs.MPG );
	} else {
		MPGBar_mc._xscale = 100;
		if ( EngineAttrs.ICERPM == 0 ){
			MPGDisplay_mc.MPG_txt.text = "Stealth";
		} else {
			MPGDisplay_mc.MPG_txt.text = "Warp";
		}
	}

	//Some tricky color transforms for the MPG bar
	if (EngineAttrs.MPG > 60 && EngineAttrs.MPG < 90)
	{
		MPGRed = 255-(255/30*(Math.round(EngineAttrs.MPG-60)));
	} else if (EngineAttrs.MPG >= 0 && EngineAttrs.MPG <= 60){
		MPGRed = 255;
	} else {
		MPGRed = 0;
	}
	if (EngineAttrs.MPG > 10 && EngineAttrs.MPG < 40)
	{
		MPGGreen = 255/30*(Math.round(EngineAttrs.MPG-10));
	} else if (EngineAttrs.MPG >= 0 && EngineAttrs.MPG <= 10){
		MPGGreen = 0;
	} else {
		MPGGreen = 255;
	}
	
	if (Math.round(EngineAttrs.MPG) == 255) {
		MPGColorTrans.alphaMultiplier = 0.4;
	} else {
		MPGColorTrans.alphaMultiplier = 1;
	}
	
	MPGColorTrans.redOffset = MPGRed;
	MPGColorTrans.greenOffset = MPGGreen;
	//MPGColorTrans.blueOffset = MPGBlue;
	MPGTrans.colorTransform = MPGColorTrans;
	
	//Offset color of the bar and apply it to the text fields.
	MPGDisplay_mc.MPG_txt.textColor = RgbColor.convertTriplet( MPGRed, MPGGreen, MPGGreen );

	if (Math.round(EngineAttrs.MPG) < 255){
		MPGDisplay_mc.Label_txt.textColor = RgbColor.convertTriplet( MPGRed, MPGGreen, MPGGreen );
	} else {
		MPGDisplay_mc.Label_txt.textColor = "0x009999";
	}

	//Set SOC bar to scale between 40 and 80
	SOCscale = (EngineAttrs.SOC - 40) * 2.5;
	if (SOCscale >= 100) {
		SOCscale = 100;
	}
	SOCBar_mc._xscale = SOCscale;
	SOCDisplay_mc.SOC_txt.text = (Math.round(EngineAttrs.SOC*10))/10 + "%";

	//Match battery meter colors from the Prius MFD
	if (EngineAttrs.SOC <= 48) {
		SOCColorTrans.rgb = 0xCC6699;
		SOCTrans.colorTransform = SOCColorTrans;
		SOCDisplay_mc.SOC_txt.textColor = "0xFF99CC";
		SOCDisplay_mc.Label_txt.textColor = "0xFF99CC";
	} else if (EngineAttrs.SOC <= 65) {
		SOCColorTrans.rgb = 0x3366CC;
		SOCTrans.colorTransform = SOCColorTrans;
		SOCDisplay_mc.SOC_txt.textColor = "0x77AAFF";
		SOCDisplay_mc.Label_txt.textColor = "0x77AAFF";
	} else {
		SOCColorTrans.rgb = 0x33AA33;
		SOCTrans.colorTransform = SOCColorTrans;
		SOCDisplay_mc.SOC_txt.textColor = "0x77EE77";
		SOCDisplay_mc.Label_txt.textColor = "0x77EE77";
	}
}

//Add a listener to the Nomograph checkbox to show/hide the line.
Overlay.selected = true;
var checkboxListener:Object = new Object();
checkboxListener.click = function(evt_obj:Object) {
	if (evt_obj.target.selected) {
		UpdateSliders();
		myLine.setActive( true );
	} else {
		myLine.setActive( false );
	}
}
Overlay.addEventListener("click", checkboxListener);

// Set up the button states and drag limits for the sliders. 
MG2Slider_mc.Handle_mc.onPress = function() {
	this.startDrag( true, 0, MG2UpperSliderLimit, 0, MG2LowerSliderLimit );
	SlidersNeedUpdate = true;
}
MG2Slider_mc.Handle_mc.onRelease = function() {
	UpdateSliders();
	SlidersNeedUpdate = false;
	this.stopDrag();
	delete this.onEnterFrame;
}
MG2Slider_mc.Handle_mc.onReleaseOutside = function() {
	UpdateSliders();
	SlidersNeedUpdate = false;
	this.stopDrag();
	delete this.onEnterFrame;
}

ICESlider_mc.Handle_mc.onPress = function() {
	this.startDrag( true, 0, ICEUpperSliderLimit, 0, 0 );
	SlidersNeedUpdate = true;
}
ICESlider_mc.Handle_mc.onRelease = function() {
	SetSafeICERPM();
	UpdateSliders();
	SlidersNeedUpdate = false;
	this.stopDrag();
	delete this.onEnterFrame;
}
ICESlider_mc.Handle_mc.onReleaseOutside = function() {
	SetSafeICERPM();
	UpdateSliders();
	SlidersNeedUpdate = false;
	this.stopDrag();
	delete this.onEnterFrame;
}

/*
Broken button stuff. Will be fixed... eventually.

ButtonGuide_mc.attachMovie("DynamicButton_mc", "Button1_mc", this.getNextHighestDepth() );
_level0.Button1_mc.ButtonBG_mc.ButtonColor.setRGB("0xFF6666"); 
_level0.Button1_mc.DynamicButton_mc.Button_txt.text = "Push Me";

function ConfigButtons() {
	DynamicButton_mc.onRelease = function() {
		//do something
	}
	_level0.DynamicButton_mc.ButtonBG_mc.ButtonColor.setRGB("0xFF6666");
	_level0.DynamicButton_mc.Button_txt.text = "Push Me";
	clearInterval(ButtonSetupInterval);
}

ButtonSetupInterval = setInterval(ConfigButtons, 50);
*/

// eof
